package handelers

import(
	"strings"
	"net/http"
	"fmt"
	"strconv"
	"encoding/json"
)

func getCountryDataByCode(countryCode string) (*CountryInfo, error) {
	url := fmt.Sprintf(URL_API + "%s", countryCode)
	resp, err := http.Get(url)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return nil, fmt.Errorf(
			ERROR_RETRIEVING_COUNTRY_DATA +
			COUNTRY_CODE_REMINDER +
			CITY_LIMIT_REMINDER +
			EXAMPLE_URL +
			ASSISTANCE_REMINDER,
			countryCode, INFO_URL_EXAMPLE, HOMEPAGE_DEFAULT)
	}
	var countries []CountryInfo
	if err := json.NewDecoder(resp.Body).Decode(&countries); err != nil {
		return nil, err
	}
	return &countries[0], nil
}

func CountryInfoHandler(w http.ResponseWriter, r *http.Request) {
	parts := strings.Split(r.URL.Path, SLASH_DEFAULT)
	countryCode := parts[4]

	country, err := getCountryDataByCode(countryCode)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	cityCount := 10 // Default
	cityCountStr := r.URL.Query().Get(CITIES) 
	if cityCountStr != "" {
		if count, err := strconv.Atoi(cityCountStr); err == nil && count > 0 {
			cityCount = count //number of cities to be displayed if specified
		}
	}

	cities, err := getCitiesByCountry(country.Name.Common)
	if err != nil {
		http.Error(w, err.Error(), http.StatusInternalServerError)
		return
	}

	if len(cities) > cityCount {
		cities = cities[:cityCount]
	}

	response := CombinedInfo{
		Name:       country.Name.Common,
		Continents: country.Continents,
		Population: country.Population,
		Languages:  country.Languages,
		Borders:    country.Borders,
		Flags:      country.Flags,
		Capital:    "",
		Cities:     strings.Join(cities, ", "),
	}

	if len(country.Capital) > 0 {
		response.Capital = country.Capital[0]
	}

	w.Header().Set(HEADER_CONTENT_TYPE, FORMAT_JSON)
	json.NewEncoder(w).Encode(response)
}

func getCitiesByCountry(countryName string) ([]string, error) {
    url := fmt.Sprintf(CITIES_URL +"%s", countryName)
    resp, err := http.Get(url)
    if err != nil {
        return nil, err
    }
    defer resp.Body.Close()

    if resp.StatusCode != http.StatusOK {
        return nil, fmt.Errorf(CITIES_ERROR + "%d", resp.StatusCode)
    }

    var responseBody map[string]interface{}
    if err := json.NewDecoder(resp.Body).Decode(&responseBody); err != nil {
        return nil, err
    }

    if cities, ok := responseBody[DATA].([]interface{}); ok {
        var cityList []string
        for _, city := range cities {
            if cityStr, ok := city.(string); ok {
                cityList = append(cityList, cityStr)
            }
        }
        return cityList, nil
    }

    return nil, fmt.Errorf(CITIES_ERROR2, countryName)
}